@@ -4,7 +4,7 @@ from contextlib import suppress
 from functools import total_ordering
 from pathlib import Path
 import re
-from typing import TYPE_CHECKING, FrozenSet, List, Tuple
+from typing import TYPE_CHECKING, FrozenSet, List, Tuple, cast
 
 from .models import PkgFolder
 from .resolvers import CurseResolver, Defn, TukuiResolver, WowiResolver
@@ -21,6 +21,11 @@ _ids_to_sources = {
     'X-Tukui-ProjectID': TukuiResolver.source,
     'X-WoWI-ID': WowiResolver.source,
 }
+_sources_to_sort_weights = {
+    CurseResolver.source: 0,
+    TukuiResolver.source: 2,
+    WowiResolver.source: 1,
+}
 
 
 @total_ordering
@@ -35,7 +40,7 @@ class AddonFolder:
     @cached_property
     def defns_from_toc(self) -> FrozenSet[Defn]:
         return frozenset(
-            Defn(s, i)
+            Defn(source=s, name=i)
             for s, i in ((s, self.toc_reader[k].value) for k, s in _ids_to_sources.items())
             if i
         )
@@ -62,24 +67,27 @@ def get_folders(manager: Manager, exclude_own: bool = True) -> FrozenSet[AddonFo
     def make_addon_folder(path: Path):
         if path.name not in own_folders and path.is_dir() and not path.is_symlink():
             with suppress(FileNotFoundError):
-                return AddonFolder(path.name, TocReader.from_path_name(path))
+                return AddonFolder(path.name, TocReader.from_parent_folder(path))
 
     if exclude_own:
-        own_folders = {f.name for f in manager.db_session.query(PkgFolder).all()}
+        own_folders = {f.name for f in manager.database.query(PkgFolder).all()}
     else:
         own_folders = set()
     return frozenset(f for f in map(make_addon_folder, manager.config.addon_dir.iterdir()) if f)
 
 
 async def match_toc_ids(manager: Manager, leftovers: FrozenSet[AddonFolder]) -> MatchGroups:
-    "Attempt to match add-ons from TOC file source ID entries."
+    "Attempt to match add-ons from TOC-file source ID entries."
 
-    def keyer(value: AddonFolder):
+    def bucket_keyer(value: AddonFolder):
         return next(d for d in defns if value.defns_from_toc & d)
 
+    def sort_keyer(value: Defn):
+        return _sources_to_sort_weights[value.source]
+
     matches = [a for a in sorted(leftovers) if a.defns_from_toc]
     defns = list(merge_intersecting_sets(a.defns_from_toc for a in matches))
-    return [(f, list(b)) for b, f in bucketise(matches, keyer).items()]
+    return [(f, sorted(b, key=sort_keyer)) for b, f in bucketise(matches, bucket_keyer).items()]
 
 
 async def match_dir_names(manager: Manager, leftovers: FrozenSet[AddonFolder]) -> MatchGroups:
@@ -90,28 +98,31 @@ async def match_dir_names(manager: Manager, leftovers: FrozenSet[AddonFolder]) -
 
     def sort_keyer(value: Tuple[FrozenSet[AddonFolder], Defn]):
         folders, defn = value
-        return (-len(folders), defn)
+        return (-len(folders), _sources_to_sort_weights[defn.source])
 
     await manager.synchronise()
 
     # We can't use an intersection here because it's not guaranteed to
-    # return ``AddonFolder``s
+    # return ``AddonFolder``s - the duck typing semantics of '&' appear
+    # to be undefined
     matches = [
-        (frozenset(l for l in leftovers if l in f), Defn(i.source, i.id))
+        (
+            frozenset(e for e in leftovers if e in cast('List[AddonFolder]', f)),
+            Defn(source=i.source, name=i.id),
+        )
         for i in manager.catalogue.__root__
         for f in i.folders
         if manager.config.game_flavour in i.compatibility and frozenset(f) <= leftovers
     ]
     folders = list(merge_intersecting_sets(f for f, _ in matches))
     return [
-        # IDs are sorted in decreasing order of number of matching folders
         (sorted(f), uniq(d for _, d in sorted(b, key=sort_keyer)))
         for f, b in bucketise(matches, bucket_keyer).items()
     ]
 
 
 async def match_toc_names(manager: Manager, leftovers: FrozenSet[AddonFolder]) -> MatchGroups:
-    "Attempt to match add-ons from TOC file name entries."
+    "Attempt to match add-ons from TOC-file name entries."
 
     def normalise(value: str):
         return re.sub(r'[^0-9A-Za-z]', '', value.casefold())
@@ -119,5 +130,5 @@ async def match_toc_names(manager: Manager, leftovers: FrozenSet[AddonFolder]) -
     await manager.synchronise()
 
     norm_to_items = bucketise(manager.catalogue.__root__, key=lambda i: normalise(i.name))
-    matches = ((l, norm_to_items.get(normalise(l.name))) for l in sorted(leftovers))
-    return [([l], uniq(Defn(i.source, i.id) for i in m)) for l, m in matches if m]
+    matches = ((e, norm_to_items.get(normalise(e.name))) for e in sorted(leftovers))
+    return [([e], uniq(Defn(source=i.source, name=i.id) for i in m)) for e, m in matches if m]
