@@ -1,14 +1,24 @@
 from __future__ import annotations
 
-from typing import Any, ClassVar, List, Type
+from datetime import datetime
+from typing import Any, ClassVar, List, Type, cast
 
 import pydantic
-from pydantic import create_model
-from sqlalchemy import exc, func, inspect
+from sqlalchemy import (
+    Column,
+    DateTime,
+    ForeignKeyConstraint,
+    Integer,
+    MetaData,
+    String,
+    UniqueConstraint,
+    and_,
+    exc,
+    func,
+    inspect,
+)
 from sqlalchemy.ext.declarative import DeclarativeMeta, as_declarative
-from sqlalchemy.orm import relationship
-from sqlalchemy.schema import Column, ForeignKeyConstraint, MetaData, UniqueConstraint
-from sqlalchemy.types import DateTime, Integer, String
+from sqlalchemy.orm import object_session, relationship
 
 
 class _BaseCoercer(pydantic.BaseModel):
@@ -30,16 +40,16 @@ class _BaseTableMeta(DeclarativeMeta):
     def __init__(cls, *args: Any) -> None:
         super().__init__(*args)
         try:
-            columns = inspect(cls).columns
+            inspector = inspect(cls)
         except exc.NoInspectionAvailable:
             pass
         else:
-            cls.Coercer = create_model(
+            cls.Coercer = pydantic.create_model(
                 f'{cls.__name__}Coercer',
                 __base__=_BaseCoercer,
                 **{
                     c.name: (c.type.python_type, ...)
-                    for c in columns
+                    for c in inspector.columns
                     if not c.foreign_keys and not c.name.startswith('_') and not c.server_default
                 },
             )
@@ -71,9 +81,31 @@ class Pkg(_BaseTable):
     download_url = Column(String, nullable=False)
     date_published = Column(DateTime, nullable=False)
     version = Column(String, nullable=False)
-    folders = relationship('PkgFolder', cascade='all, delete-orphan', backref='pkg')
-    options = relationship('PkgOptions', cascade='all, delete-orphan', uselist=False)
-    deps = relationship('PkgDep', cascade='all, delete-orphan', backref='pkg')
+    folders = cast(
+        'List[PkgFolder]', relationship('PkgFolder', cascade='all, delete-orphan', backref='pkg')
+    )
+    options = cast(
+        'PkgOptions', relationship('PkgOptions', cascade='all, delete-orphan', uselist=False)
+    )
+    deps = cast(
+        'List[PkgDep]', relationship('PkgDep', cascade='all, delete-orphan', backref='pkg')
+    )
+
+    @property
+    def logged_versions(self) -> List[PkgVersionLog]:
+        session: Any = object_session(self)
+        return (
+            (
+                session.query(PkgVersionLog)
+                .filter(
+                    and_(PkgVersionLog.pkg_source == self.source, PkgVersionLog.pkg_id == self.id)
+                )
+                .order_by(PkgVersionLog.install_time.desc())
+                .all()
+            )
+            if session
+            else []
+        )
 
 
 class PkgFolder(_BaseTable):
@@ -116,18 +148,41 @@ class PkgVersionLog(_BaseTable):
     pkg_id = Column(String, nullable=False, primary_key=True)
 
 
-class _PkgModel(Pkg.Coercer):
-    folders: List[PkgFolder.Coercer]
-    options: PkgOptions.Coercer
-    deps: List[PkgDep.Coercer]
+class _PkgFolder(_BaseCoercer):
+    name: str
+
+
+class _PkgOptions(_BaseCoercer):
+    strategy: str
+
+
+class _PkgDep(_BaseCoercer):
+    id: str
+
+
+class _PkgVersion(_BaseCoercer):
+    version: str
+    install_time: datetime
+
+
+class PkgModel(_BaseCoercer):
+    source: str
+    id: str
+    slug: str
+    name: str
+    description: str
+    url: str
+    download_url: str
+    date_published: datetime
+    version: str
+    folders: List[_PkgFolder]
+    options: _PkgOptions
+    deps: List[_PkgDep]
+    logged_versions: List[_PkgVersion]
 
 
 class MultiPkgModel(pydantic.BaseModel):
-    __root__: List[_PkgModel]
-
-    @classmethod
-    def from_orm(cls, obj: List[Pkg]) -> MultiPkgModel:
-        return cls(__root__=[_PkgModel.from_orm(v) for v in obj])
+    __root__: List[PkgModel]
 
 
 def should_migrate(engine: Any, version: str) -> bool:
@@ -137,14 +192,13 @@ def should_migrate(engine: Any, version: str) -> bool:
     with engine.begin() as conn:
         try:
             current = conn.execute(
-                'SELECT version_num FROM alembic_version WHERE version_num = (?)', version,
+                'SELECT version_num FROM alembic_version WHERE version_num = (?)',
+                version,
             ).scalar()
         except exc.OperationalError:
             return True
         else:
-            if not current:
-                return True
-    return False
+            return not current
 
 
 def is_pkg(value: Any) -> bool:
