@@ -10,10 +10,9 @@ from typing import (
     AsyncIterable,
     ClassVar,
     Dict,
-    Iterable,
     List,
-    NamedTuple,
     Optional as O,
+    Sequence,
     Set,
     Tuple,
     cast,
@@ -23,15 +22,7 @@ from pydantic import BaseModel
 from yarl import URL
 
 from . import exceptions as E, models as m
-from .utils import (
-    Literal,
-    ManagerAttrAccessMixin,
-    cached_property,
-    gather,
-    run_in_thread as t,
-    slugify,
-    uniq,
-)
+from .utils import Literal, cached_property, gather, run_in_thread as t, slugify, uniq
 
 if TYPE_CHECKING:
     from .manager import Manager
@@ -40,71 +31,77 @@ if TYPE_CHECKING:
 
 
 class Strategies(enum.Enum):
-    default = enum.auto()
-    latest = enum.auto()
-    curse_latest_beta = enum.auto()
-    curse_latest_alpha = enum.auto()
-    any_flavour = enum.auto()
-    version = enum.auto()
-
-    @classmethod
-    def exposed(cls) -> Iterable[Strategies]:
-        return (s for s in cls if s not in {cls.default, cls.version})
+    default = 'default'
+    latest = 'latest'
+    curse_latest_beta = 'curse_latest_beta'
+    curse_latest_alpha = 'curse_latest_alpha'
+    any_flavour = 'any_flavour'
+    version = 'version'
 
 
-class Defn(NamedTuple):
+class Defn(BaseModel):
     source: str
+    source_id: O[str] = None
     name: str
     strategy: Strategies = Strategies.default
     strategy_vals: Tuple[str, ...] = ()
-    source_id: O[str] = None
 
     @classmethod
     def from_pkg(cls, pkg: m.Pkg) -> Defn:
-        strategy = Strategies[pkg.options.strategy]
-        strategy_vals = (pkg.version,) if strategy is Strategies.version else ()
-        return cls(pkg.source, pkg.slug, strategy, strategy_vals, pkg.id)
+        defn = cls(
+            source=pkg.source, source_id=pkg.id, name=pkg.slug, strategy=pkg.options.strategy
+        )
+        if defn.strategy is Strategies.version:
+            defn.strategy_vals = (pkg.version,)
+        return defn
 
-    def with_name(self, name: str) -> Defn:
-        return self.__class__(self.source, name, self.strategy, self.strategy_vals, self.source_id)
+    @classmethod
+    def get(cls, source: str, name: str) -> Defn:
+        return cls(source=source, name=name)
 
-    def with_strategy(self, strategy: Strategies) -> Defn:
-        return self.__class__(self.source, self.name, strategy, (), self.source_id)
+    def with_(self, **kwargs: Any) -> Defn:
+        return self.__class__(**{**self.__dict__, **kwargs})
 
     def with_version(self, version: str) -> Defn:
-        return self.__class__(
-            self.source, self.name, Strategies.version, (version,), self.source_id
-        )
+        return self.with_(strategy='version', strategy_vals=(version,))
+
+    def __eq__(self, other: Any) -> bool:
+        if isinstance(other, self.__class__):
+            other = tuple(other.__dict__.values())
+        return tuple(self.__dict__.values()) == other
+
+    def __hash__(self) -> int:
+        return hash(tuple(self.__dict__.values()))
 
     def __str__(self) -> str:
         return f'{self.source}:{self.name}'
 
 
-class _CItem(BaseModel):
+class _CatalogueEntry(BaseModel):
     source: str
     id: str
     slug: str = ''
     name: str
-    compatibility: List[Literal['retail', 'classic']]
+    compatibility: Set[Literal['retail', 'classic']] = {'retail', 'classic'}
     folders: List[List[str]] = []
 
 
 class MasterCatalogue(BaseModel):
-    __root__: List[_CItem]
+    __root__: List[_CatalogueEntry]
 
     class Config:
-        keep_untouched: Any = (cached_property,)
+        keep_untouched = (cast(Any, cached_property),)
 
     @classmethod
     async def collate(cls) -> MasterCatalogue:
         from types import SimpleNamespace
 
-        from .manager import Manager, init_web_client
+        from .manager import init_web_client
 
         resolvers = (CurseResolver, WowiResolver, TukuiResolver)
 
         async with init_web_client() as web_client:
-            faux_manager = cast(Manager, SimpleNamespace(web_client=web_client))
+            faux_manager = cast('Manager', SimpleNamespace(web_client=web_client))
             items = [a for r in resolvers async for a in r(faux_manager).collect_items()]
         catalogue = cls(__root__=items)
         return catalogue
@@ -114,7 +111,7 @@ class MasterCatalogue(BaseModel):
         return {a.slug: a.id for a in self.__root__ if a.source == 'curse'}
 
 
-class Resolver(ManagerAttrAccessMixin):
+class Resolver:
     source: ClassVar[str]
     name: ClassVar[str]
     strategies: ClassVar[Set[Strategies]]
@@ -123,9 +120,7 @@ class Resolver(ManagerAttrAccessMixin):
         self.manager = manager
 
     def __init_subclass__(cls) -> None:
-        async def resolve_wrapper(
-            self: Resolver, defn: Defn, metadata: O[JsonDict] = None
-        ) -> m.Pkg:
+        async def resolve_wrapper(self: Resolver, defn: Defn, metadata: O[JsonDict]) -> m.Pkg:
             if defn.strategy in self.strategies:
                 return await resolve_one(self, defn, metadata)  # type: ignore
             raise E.PkgStrategyUnsupported(defn.strategy)
@@ -142,16 +137,16 @@ class Resolver(ManagerAttrAccessMixin):
     def get_name_from_url(value: str) -> O[str]:
         "Attempt to extract a definition name from a given URL."
 
-    async def resolve(self, defns: List[Defn]) -> Dict[Defn, Any]:
+    async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
         "Resolve add-on definitions into packages."
-        results = await gather(self.resolve_one(d) for d in defns)
+        results = await gather(self.resolve_one(d, None) for d in defns)
         return dict(zip(defns, results))
 
-    async def resolve_one(self, defn: Defn, metadata: O[JsonDict] = None) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: O[JsonDict]) -> m.Pkg:
         "Resolve an individual definition into a package."
         raise NotImplementedError
 
-    async def collect_items(self) -> AsyncIterable[_CItem]:
+    async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
         "Yield add-ons from source for cataloguing."
         return
         yield
@@ -184,16 +179,27 @@ class CurseResolver(Resolver):
         ):
             return url.parts[3].lower()
 
-    async def resolve(self, defns: List[Defn]) -> Dict[Defn, Any]:
+    async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
+        from aiohttp import ClientResponseError
+
+        from .manager import cache_json_response
+
         ids_for_defns = {
-            d: (d.source_id or self.catalogue.curse_slugs.get(d.name) or d.name) for d in defns
+            d: (d.source_id or self.manager.catalogue.curse_slugs.get(d.name) or d.name)
+            for d in defns
         }
         numeric_ids = {i for i in ids_for_defns.values() if i.isdigit()}
-        async with self.web_client.post(self.addon_api_url, json=list(numeric_ids)) as response:
-            if response.status == 404:
-                json_response = []
-            else:
-                json_response = await response.json()
+        try:
+            json_response = await cache_json_response(
+                self.manager,
+                self.addon_api_url,
+                300,
+                request_kwargs={'method': 'POST', 'json': list(numeric_ids)},
+            )
+        except ClientResponseError as error:
+            if error.status != 404:
+                raise
+            json_response = []
 
         api_results = {str(r['id']): r for r in json_response}
         results = await gather(
@@ -236,7 +242,7 @@ class CurseResolver(Resolver):
 
             else:
                 classic_version_prefix = '1.13'
-                flavour = 'wow_classic' if self.config.is_classic else 'wow_retail'
+                flavour = 'wow_classic' if self.manager.config.is_classic else 'wow_retail'
 
                 def supports_game_version(f: JsonDict):
                     # Files can belong both to retail and classic
@@ -245,35 +251,33 @@ class CurseResolver(Resolver):
                     # ``gameVersion`` might not be populated so we still have to
                     # check the value of ``gameVersionFlavor``
                     return f['gameVersionFlavor'] == flavour or any(
-                        v.startswith(classic_version_prefix) is self.config.is_classic
+                        v.startswith(classic_version_prefix) is self.manager.config.is_classic
                         for v in f['gameVersion']
                     )
 
             # 1 = stable; 2 = beta; 3 = alpha
             if defn.strategy is Strategies.latest:
 
-                def is_of_specified_quality(f: JsonDict):
+                def has_release_type(f: JsonDict):
                     return True
 
             elif defn.strategy is Strategies.curse_latest_beta:
 
-                def is_of_specified_quality(f: JsonDict):
+                def has_release_type(f: JsonDict):
                     return f['releaseType'] == 2
 
             elif defn.strategy is Strategies.curse_latest_alpha:
 
-                def is_of_specified_quality(f: JsonDict):
+                def has_release_type(f: JsonDict):
                     return f['releaseType'] == 3
 
             else:
 
-                def is_of_specified_quality(f: JsonDict):
+                def has_release_type(f: JsonDict):
                     return f['releaseType'] == 1
 
             def is_compatible(f: JsonDict):
-                return (
-                    is_not_libless(f) and supports_game_version(f) and is_of_specified_quality(f)
-                )
+                return is_not_libless(f) and supports_game_version(f) and has_release_type(f)
 
         if not files:
             raise E.PkgFileUnavailable('no files available for download')
@@ -285,7 +289,7 @@ class CurseResolver(Resolver):
             )
         except ValueError:
             raise E.PkgFileUnavailable(
-                f'no files compatible with {self.config.game_flavour} '
+                f'no files compatible with {self.manager.config.game_flavour} '
                 f'using {defn.strategy.name!r} strategy'
             )
 
@@ -311,7 +315,7 @@ class CurseResolver(Resolver):
             deps=deps,
         )
 
-    async def collect_items(self) -> AsyncIterable[_CItem]:
+    async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
         classic_version_prefix = '1.13'
         flavours = ('retail', 'classic')
 
@@ -327,15 +331,12 @@ class CurseResolver(Resolver):
                     yield c
 
         step = 1000
+        sort_order = '3'  # Alphabetical
         for index in count(0, step):
             url = (self.addon_api_url / 'search').with_query(
-                # fmt: off
-                gameId='1',
-                sort='3', # Alphabetical
-                pageSize=step, index=index,
-                # fmt: on
+                gameId='1', sort=sort_order, pageSize=step, index=index
             )
-            async with self.web_client.get(url) as response:
+            async with self.manager.web_client.get(url) as response:
                 json_response = await response.json()
 
             if not json_response:
@@ -346,13 +347,13 @@ class CurseResolver(Resolver):
                     for f in item['latestFiles']
                     if not f['exposeAsAlternative']
                 )
-                yield _CItem(
+                yield _CatalogueEntry(
                     source=self.source,
                     id=item['id'],
                     slug=item['slug'],
                     name=item['name'],
+                    compatibility=set(excise_compatibility(item['latestFiles'])),
                     folders=folders,
-                    compatibility=list(excise_compatibility(item['latestFiles'])),
                 )
 
 
@@ -386,39 +387,42 @@ class WowiResolver(Resolver):
             and url.parts[1] == 'downloads'
         ):
             if url.name == 'landing.php':
-                id_ = url.query.get('fileid')
-                if id_:
-                    return id_
+                source_id = url.query.get('fileid')
+                if source_id:
+                    return source_id
             elif url.name == 'fileinfo.php':
-                id_ = url.query.get('id')
-                if id_:
-                    return id_
+                source_id = url.query.get('id')
+                if source_id:
+                    return source_id
             else:
                 match = re.match(r'^(?:download|info)(?P<id>\d+)', url.name)
                 if match:
                     return match.group('id')
 
-    async def resolve(self, defns: List[Defn]) -> Dict[Defn, Any]:
-        if self._files is None:
-            from .manager import cache_json_response
+    async def resolve(self, defns: Sequence[Defn]) -> Dict[Defn, Any]:
+        from aiohttp import ClientResponseError
 
-            files = await cache_json_response(
-                self.manager,
-                self.list_api_url,
-                3600,
-                label=f'Synchronising {self.name} catalogue',
-            )
-            self._files = {i['UID']: i for i in files}
+        from .manager import cache_json_response
+
+        async with self.manager.locks['load WoWI catalogue']:
+            if self._files is None:
+                files = await cache_json_response(
+                    self.manager,
+                    self.list_api_url,
+                    3600,
+                    label=f'Synchronising {self.name} catalogue',
+                )
+                self._files = {i['UID']: i for i in files}
 
         ids_for_defns = {d: ''.join(takewhile(str.isdigit, d.name)) for d in defns}
         numeric_ids = {i for i in ids_for_defns.values() if i.isdigit()}
-
         url = self.details_api_url / f'{",".join(numeric_ids)}.json'
-        async with self.web_client.get(url) as response:
-            if response.status == 404:
-                json_response = []
-            else:
-                json_response = await response.json()
+        try:
+            json_response = await cache_json_response(self.manager, url, 300)
+        except ClientResponseError as error:
+            if error.status != 404:
+                raise
+            json_response = []
 
         api_results = {r['UID']: {**self._files[r['UID']], **r} for r in json_response}
         results = await gather(
@@ -445,16 +449,16 @@ class WowiResolver(Resolver):
             options=m.PkgOptions(strategy=defn.strategy.name),
         )
 
-    async def collect_items(self) -> AsyncIterable[_CItem]:
-        async with self.web_client.get(self.list_api_url) as response:
+    async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
+        async with self.manager.web_client.get(self.list_api_url) as response:
             json_response = await response.json()
+
         for item in json_response:
-            yield _CItem(
+            yield _CatalogueEntry(
                 source=self.source,
                 id=item['UID'],
                 name=item['UIName'],
                 folders=[item['UIDir']],
-                compatibility=['retail', 'classic'],
             )
 
 
@@ -479,12 +483,12 @@ class TukuiResolver(Resolver):
             if name:
                 return name
 
-    async def resolve_one(self, defn: Defn, metadata: ... = None) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: Any) -> m.Pkg:
         name = ui_name = self.retail_uis.get(defn.name)
         if not name:
             name = ''.join(takewhile('-'.__ne__, defn.name))
 
-        if self.config.is_classic:
+        if self.manager.config.is_classic:
             query = 'classic-addon'
         elif ui_name:
             query = 'ui'
@@ -492,7 +496,7 @@ class TukuiResolver(Resolver):
             query = 'addon'
 
         url = self.api_url.with_query({query: name})
-        async with self.web_client.get(url) as response:
+        async with self.manager.web_client.get(url) as response:
             if not response.content_length:
                 raise E.PkgNonexistent
             addon = await response.json(content_type=None)  # text/html
@@ -510,32 +514,31 @@ class TukuiResolver(Resolver):
             options=m.PkgOptions(strategy=defn.strategy.name),
         )
 
-    async def collect_items(self) -> AsyncIterable[_CItem]:
-        for query, param in [
-            ('ui', 'tukui'),
-            ('ui', 'elvui'),
-            ('addons', 'all'),
-            ('classic-addons', 'all'),
+    async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
+        for query, param, compatibility in [
+            ('ui', 'tukui', 'retail'),
+            ('ui', 'elvui', 'retail'),
+            ('addons', 'all', 'retail'),
+            ('classic-addons', 'all', 'classic'),
         ]:
             url = self.api_url.with_query({query: param})
-            async with self.web_client.get(url) as response:
+            async with self.manager.web_client.get(url) as response:
                 metadata = await response.json(content_type=None)  # text/html
 
             if query == 'ui':
-                yield _CItem(
+                yield _CatalogueEntry(
                     source=self.source,
                     id=metadata['id'],
                     name=metadata['name'],
-                    compatibility=('retail',),
+                    compatibility={compatibility},
                 )
             else:
-                compatibility = ('retail' if query == 'addons' else 'classic',)
                 for item in metadata:
-                    yield _CItem(
+                    yield _CatalogueEntry(
                         source=self.source,
                         id=item['id'],
                         name=item['name'],
-                        compatibility=compatibility,
+                        compatibility={compatibility},
                     )
 
 
@@ -555,7 +558,7 @@ class GithubResolver(Resolver):
         if url.host == 'github.com' and len(url.parts) > 2:
             return '/'.join(url.parts[1:3])
 
-    async def resolve_one(self, defn: Defn, metadata: ... = None) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: Any) -> m.Pkg:
         """Resolve a hypothetical add-on hosted on GitHub.
 
         The GitHub resolver, ahem, 'builds' on the work done by Torkus
@@ -584,7 +587,7 @@ class GithubResolver(Resolver):
             release_url = repo_url / 'releases/tags' / defn.strategy_vals[0]
         else:
             release_url = repo_url / 'releases/latest'
-        async with self.web_client.get(release_url) as response:
+        async with self.manager.web_client.get(release_url) as response:
             if response.status == 404:
                 raise E.PkgFileUnavailable('release not found')
             release_metadata = await response.json()
@@ -610,7 +613,7 @@ class GithubResolver(Resolver):
                         a
                         for a in assets
                         if is_valid_asset(a)
-                        and a['name'].endswith('-classic.zip') is self.config.is_classic
+                        and a['name'].endswith('-classic.zip') is self.manager.config.is_classic
                     ),
                     filter(is_valid_asset, assets),
                 )
@@ -637,27 +640,30 @@ class InstawowResolver(Resolver):
     name = 'instawow'
     strategies = {Strategies.default}
 
-    _addons = {('0', 'weakauras-companion'), ('1', 'weakauras-companion-autoupdate')}
+    _addons = {
+        ('0', 'weakauras-companion'),
+        ('1', 'weakauras-companion-autoupdate'),
+    }
 
-    async def resolve_one(self, defn: Defn, metadata: ... = None) -> m.Pkg:
+    async def resolve_one(self, defn: Defn, metadata: Any) -> m.Pkg:
         try:
-            id_, slug = next(p for p in self._addons if defn.name in p)
+            source_id, slug = next(p for p in self._addons if defn.name in p)
         except StopIteration:
             raise E.PkgNonexistent
 
-        from .wa_updater import WaCompanionBuilder, WaConfigError
+        from .wa_updater import BuilderConfig, WaCompanionBuilder
 
-        builder = WaCompanionBuilder(self.manager)
-        if id_ == '1':
-            try:
-                await builder.build()
-            except WaConfigError:
-                raise E.PkgFileUnavailable('account named not provided')
+        sentinel = '__sentinel__'
+        builder = WaCompanionBuilder(self.manager, BuilderConfig(account=sentinel))
+        if source_id == '1':
+            if builder.builder_config.account == sentinel:
+                raise E.PkgFileUnavailable('account name not provided')
+            await builder.build()
 
         checksum = await t(builder.checksum)()
         return m.Pkg(
             source=self.source,
-            id=id_,
+            id=source_id,
             slug=slug,
             name='WeakAuras Companion',
             description='A WeakAuras Companion clone.',
