@@ -4,10 +4,13 @@ import asyncio
 from collections import defaultdict, deque
 from contextlib import contextmanager
 from datetime import datetime, timedelta
-from functools import partial
-from itertools import chain, repeat
-from pathlib import Path
+from functools import partial, wraps
+from itertools import chain, repeat, takewhile
+from pathlib import Path, PurePath
+import posixpath
 import re
+from shutil import move as _move
+from tempfile import mkdtemp
 from typing import (
     TYPE_CHECKING,
     AbstractSet,
@@ -15,6 +18,7 @@ from typing import (
     AsyncIterator,
     Awaitable,
     Callable,
+    DefaultDict,
     Dict,
     Generic,
     Hashable,
@@ -24,6 +28,7 @@ from typing import (
     NamedTuple,
     Optional,
     Sequence,
+    Set,
     Tuple,
     TypeVar,
     Union,
@@ -32,26 +37,20 @@ from typing import (
 )
 
 try:
-    from typing import Literal as _Literal
+    from typing import Literal
 except ImportError:
-    from typing_extensions import Literal as _Literal
-Literal = _Literal  # ...
+    from typing_extensions import Literal
 
 if TYPE_CHECKING:
     from prompt_toolkit.shortcuts import ProgressBar
 
-    from .manager import CliManager
-
     _H = TypeVar('_H', bound=Hashable)
     _AnySet = TypeVar('_AnySet', bound=AbstractSet)
+
+
 _V = TypeVar('_V')
 
 
-class ManagerAttrAccessMixin:
-    def __getattr__(self, name: str) -> Any:
-        return getattr(self.manager, name)
-
-
 class _TocEntry(NamedTuple):
     key: str
     value: str
@@ -63,6 +62,8 @@ class _TocEntry(NamedTuple):
 class TocReader:
     """Extracts key–value pairs from TOC files."""
 
+    default = ''
+
     def __init__(self, contents: str) -> None:
         possible_entries = (
             map(str.strip, e.lstrip('#').partition(':')[::2])
@@ -70,7 +71,6 @@ class TocReader:
             if e.startswith('##')
         )
         self.entries = {k: v for k, v in possible_entries if k}
-        self.default = ''
 
     def __getitem__(self, key: Union[str, Tuple[str, ...]]) -> _TocEntry:
         if isinstance(key, tuple):
@@ -85,7 +85,7 @@ class TocReader:
         return cls(path.read_text(encoding='utf-8-sig', errors='replace'))
 
     @classmethod
-    def from_path_name(cls, path: Path) -> TocReader:
+    def from_parent_folder(cls, path: Path) -> TocReader:
         return cls.from_path(path / f'{path.name}.toc')
 
 
@@ -103,13 +103,13 @@ class cached_property(Generic[_V]):
 
     def __get__(self, o: Any, t: Optional[type] = None) -> Union[cached_property[_V], _V]:
         if o is None:
-            return self.f
+            return self
         else:
             o.__dict__[self.f.__name__] = v = self.f(o)
             return v
 
 
-def bucketise(iterable: Iterable[_V], key: Callable[[_V], _H] = lambda v: v) -> Dict[_H, List[_V]]:
+def bucketise(iterable: Iterable[_V], key: Callable[[_V], _H]) -> DefaultDict[_H, List[_V]]:
     "Place the elements of an iterable in a bucket according to ``key``."
     bucket: Any = defaultdict(list)
     for value in iterable:
@@ -117,9 +117,9 @@ def bucketise(iterable: Iterable[_V], key: Callable[[_V], _H] = lambda v: v) ->
     return bucket
 
 
-def dict_chain(
-    keys: Iterable[_H], default: Any, *overrides: Iterable[Tuple[_H, _V]]
-) -> Dict[_H, _V]:
+def chain_dict(
+    keys: Iterable[_H], default: Any, *overrides: Iterable[Tuple[_H, Any]]
+) -> Dict[_H, Any]:
     "Construct a dictionary from a series of iterables with overlapping keys."
     return dict(chain(zip(keys, repeat(default)), *overrides))
 
@@ -137,10 +137,11 @@ def merge_intersecting_sets(it: Iterable[_AnySet]) -> Iterable[_AnySet]:
             this_set = many_sets.popleft()
         except IndexError:
             return
+
         while True:
             for other_set in many_sets:
                 if this_set & other_set:
-                    # The in-place operator will mutate unfrozen sets
+                    # The in-place operator would mutate unfrozen sets
                     # in the original collection
                     this_set = this_set | other_set  # type: ignore
                     many_sets.remove(other_set)
@@ -150,11 +151,26 @@ def merge_intersecting_sets(it: Iterable[_AnySet]) -> Iterable[_AnySet]:
         yield this_set
 
 
-async def gather(it: Iterable[Awaitable[_V]], return_exceptions: bool = True) -> List[_V]:
+@overload
+async def gather(
+    it: Iterable[Awaitable[_V]], return_exceptions: Literal[True] = ...
+) -> List[Union[BaseException, _V]]:
+    ...
+
+
+@overload
+async def gather(it: Iterable[Awaitable[_V]], return_exceptions: Literal[False] = ...) -> List[_V]:
+    ...
+
+
+async def gather(
+    it: Iterable[Awaitable[_V]], return_exceptions: bool = True
+) -> Union[List[Union[BaseException, _V]], List[_V]]:
     return await asyncio.gather(*it, return_exceptions=return_exceptions)
 
 
 def run_in_thread(fn: Callable[..., _V]) -> Callable[..., Awaitable[_V]]:
+    @wraps(fn)
     def wrapper(*args: Any, **kwargs: Any):
         loop = asyncio.get_running_loop()
         return loop.run_in_executor(None, lambda: fn(*args, **kwargs))
@@ -205,8 +221,10 @@ def copy_resources(*packages: str) -> Iterator[Path]:
                 filename = parent_dir / resource
                 # PyOxidizer does not expose Python source files to `importlib`
                 # (see https://github.com/indygreg/PyOxidizer/issues/237).
-                # Migrations have a "null" extension added to them that is shorn
-                # from the extracted filename
+                # The filenames of Python files have a secondary ".0" extension
+                # so that PyOx will regard them as binary resources.
+                # The extension is then removed when they're copied into
+                # the temporary folder
                 if filename.suffix == '.0':
                     filename = filename.with_suffix('')
                 filename.write_bytes(read_binary(package, resource))
@@ -261,12 +279,13 @@ def make_progress_bar(**kwargs: Any) -> ProgressBar:
     # instawow will hang.  This is my ham-fisted attempt to work
     # around all that by signalling to the daemon thread to kill the app.
 
+    _SIGWINCH = getattr(signal, "SIGWINCH", None)
+
     class PatchedProgressBar(ProgressBar):
         def __exit__(self, *args: Any):
             if self._has_sigwinch:
-                self._loop.add_signal_handler(
-                    signal.SIGWINCH, self._previous_winch_handler  # type: ignore
-                )
+                self._loop.remove_signal_handler(_SIGWINCH)
+                signal.signal(_SIGWINCH, self._previous_winch_handler)
 
             if self._thread is not None:
 
@@ -277,6 +296,8 @@ def make_progress_bar(**kwargs: Any) -> ProgressBar:
                 self.app.on_invalidate = Event(self.app, attempt_exit)
                 self._thread.join()
 
+            self._app_loop.close()
+
     class DownloadProgress(formatters.Progress):
         template = formatters.Progress.template + 'MB'
 
@@ -285,7 +306,8 @@ def make_progress_bar(**kwargs: Any) -> ProgressBar:
                 return f'{value / 2 ** 20:.1f}'
 
             return HTML(self.template).format(
-                current=format_pct(progress.items_completed), total=format_pct(progress.total)
+                current=format_pct(progress.items_completed),
+                total=format_pct(progress.total) if progress.total else '?',
             )
 
     f = [
@@ -306,6 +328,20 @@ def make_progress_bar(**kwargs: Any) -> ProgressBar:
     return progress_bar
 
 
+def move(src: PurePath, dst: PurePath) -> PurePath:
+    # See https://bugs.python.org/issue32689
+    return _move(str(src), dst)
+
+
+def trash(paths: Sequence[PurePath], *, dst: PurePath, missing_ok: bool = False) -> None:
+    parent_folder = Path(mkdtemp(dir=dst, prefix=f'deleted-{paths[0].name}-'))
+    for path in paths:
+        try:
+            move(path, dst=parent_folder)
+        except (FileNotFoundError if missing_ok else ()):
+            pass
+
+
 def shasum(*values: str) -> str:
     "Base-16-encode a string using SHA-256 truncated to 32 characters."
     from hashlib import sha256
@@ -331,47 +367,66 @@ def get_version() -> str:
         return __version__
 
 
-def is_outdated(manager: CliManager) -> bool:
-    """Check on PyPI to see if the installed instawow is outdated.
+def is_outdated() -> Tuple[bool, str]:
+    """Check on PyPI to see if the installed copy of instawow is outdated.
 
     The response is cached for 24 hours.
     """
     __version__ = get_version()
     if 'dev' in __version__:
-        return False
+        return (False, '')
+
+    from aiohttp.client import ClientError
+
+    from .config import Config
+    from .manager import init_web_client
 
     def parse_version(version: str) -> Tuple[int, ...]:
-        return tuple(map(int, version.split('.')[:3]))
+        version_numbers = version.split('.')[:3]
+        int_only_version_numbers = (
+            int(''.join(takewhile('0123456789'.__contains__, e))) for e in version_numbers
+        )
+        return tuple(int_only_version_numbers)
 
-    cache_file = manager.config.temp_dir / '.pypi_version'
+    dummy_config = Config.get_dummy_config()
+    if not dummy_config.auto_update_check:
+        return (False, '')
+
+    cache_file = dummy_config.temp_dir / '.pypi_version'
     if is_not_stale(cache_file, 1, 'days'):
         version = cache_file.read_text(encoding='utf-8')
     else:
-        from aiohttp.client import ClientError
 
         async def get_metadata():
             api_url = 'https://pypi.org/pypi/instawow/json'
-            async with manager.web_client.get(api_url) as response:
+            async with init_web_client() as web_client, web_client.get(api_url) as response:
                 return await response.json()
 
+        loop = asyncio.new_event_loop()
         try:
-            version = manager.run(get_metadata())['info']['version']
+            version = loop.run_until_complete(get_metadata())['info']['version']
         except ClientError:
             version = __version__
         else:
             cache_file.write_text(version, encoding='utf-8')
+        finally:
+            loop.close()
 
-    return parse_version(version) > parse_version(__version__)
+    return (parse_version(version) > parse_version(__version__), version)
 
 
-def setup_logging(logger_dir: Path, level: Union[int, str] = 'INFO') -> int:
-    from loguru import logger
+def find_zip_base_dirs(names: Sequence[str]) -> Set[str]:
+    "Find top-level folders in a list of ZIP member paths."
+    return {n for n in (posixpath.dirname(n) for n in names) if n and posixpath.sep not in n}
 
-    handler = {
-        'sink': logger_dir / 'error.log',
-        'level': level,
-        'rotation': '1 MB',
-        'enqueue': True,
-    }
-    (handler_id,) = logger.configure(handlers=(handler,))
-    return handler_id
+
+def make_zip_member_filter(base_dirs: Set[str]) -> Callable[[str], bool]:
+    """Filter out items which are not sub-paths of top-level folders for the
+    purpose of extracting ZIP files.
+    """
+
+    def is_subpath(name: str):
+        head, sep, _ = name.partition(posixpath.sep)
+        return cast(bool, sep) and head in base_dirs
+
+    return is_subpath
