@@ -5,6 +5,11 @@ from instawow.models import Pkg
 from instawow.resolvers import Defn, Strategies
 
 
+@pytest.fixture(autouse=True)
+def mock(mock_all):
+    pass
+
+
 @pytest.mark.asyncio
 @pytest.mark.parametrize(
     'strategy', [Strategies.default, Strategies.latest, Strategies.any_flavour]
@@ -12,10 +17,10 @@ from instawow.resolvers import Defn, Strategies
 async def test_resolve_curse_simple_pkgs(manager, request, strategy):
     results = await manager.resolve(
         [
-            Defn('curse', 'tomcats', strategy),
-            Defn('curse', 'mythic-dungeon-tools', strategy),
-            Defn('curse', 'classiccastbars', strategy),
-            Defn('curse', 'elkbuffbars', strategy),
+            Defn(source='curse', name='tomcats', strategy=strategy),
+            Defn(source='curse', name='mythic-dungeon-tools', strategy=strategy),
+            Defn(source='curse', name='classiccastbars', strategy=strategy),
+            Defn(source='curse', name='elkbuffbars', strategy=strategy),
         ]
     )
     separate, retail_only, classic_only, flavour_explosion = results.values()
@@ -52,16 +57,16 @@ async def test_resolve_curse_simple_pkgs(manager, request, strategy):
 
 @pytest.mark.asyncio
 async def test_resolve_curse_latest_pkg(manager):
-    (latest_pkg,) = (await manager.resolve([Defn('curse', 'tomcats', Strategies.latest)])).values()
+    (latest_pkg,) = (
+        await manager.resolve([Defn(source='curse', name='tomcats', strategy=Strategies.latest)])
+    ).values()
     assert latest_pkg.options.strategy == 'latest'
 
 
 @pytest.mark.asyncio
 async def test_resolve_curse_versioned_pkg(manager):
     (versioned_pkg,) = (
-        await manager.resolve(
-            [Defn('curse', 'molinari', Strategies.version, ('70300.51-Release',))]
-        )
+        await manager.resolve([Defn.get('curse', 'molinari').with_version('70300.51-Release')])
     ).values()
     assert (
         versioned_pkg.options.strategy == 'version' and versioned_pkg.version == '70300.51-Release'
@@ -73,7 +78,7 @@ async def test_resolve_curse_deps(manager):
     if manager.config.is_classic:
         pytest.skip('no classic equivalent')
 
-    defns = [Defn('curse', 'mechagon-rare-share', Strategies.default)]
+    defns = [Defn(source='curse', name='mechagon-rare-share', strategy=Strategies.default)]
     with_deps = await manager.resolve(defns, with_deps=True)
     assert ['mechagon-rare-share', 'rare-share'] == [d.slug for d in with_deps.values()]
 
@@ -81,7 +86,10 @@ async def test_resolve_curse_deps(manager):
 @pytest.mark.asyncio
 async def test_resolve_wowi_pkgs(manager):
     results = await manager.resolve(
-        [Defn('wowi', '13188-molinari'), Defn('wowi', '13188', Strategies.latest)]
+        [
+            Defn(source='wowi', name='13188-molinari'),
+            Defn(source='wowi', name='13188', strategy=Strategies.latest),
+        ]
     )
     either, invalid = results.values()
 
@@ -96,10 +104,10 @@ async def test_resolve_wowi_pkgs(manager):
 async def test_resolve_tukui_pkgs(manager):
     results = await manager.resolve(
         [
-            Defn('tukui', '1'),
-            Defn('tukui', '-1'),
-            Defn('tukui', 'tukui'),
-            Defn('tukui', '1', Strategies.latest),
+            Defn(source='tukui', name='1'),
+            Defn(source='tukui', name='-1'),
+            Defn(source='tukui', name='tukui'),
+            Defn(source='tukui', name='1', strategy=Strategies.latest),
         ]
     )
     either, retail_id, retail_slug, invalid = results.values()
@@ -123,12 +131,22 @@ async def test_resolve_tukui_pkgs(manager):
 async def test_resolve_github_pkgs(manager):
     results = await manager.resolve(
         [
-            Defn('github', 'AdiAddons/AdiButtonAuras'),
-            Defn('github', 'AdiAddons/AdiButtonAuras', Strategies.version, ('2.1.0',)),
-            Defn('github', 'AdiAddons/AdiButtonAuras', Strategies.version, ('2.0.19',)),
-            Defn('github', 'WeakAuras/WeakAuras2'),
-            Defn('github', 'p3lim-wow/Molinari'),
-            Defn('github', 'layday/foo-bar'),
+            Defn(source='github', name='AdiAddons/AdiButtonAuras'),
+            Defn(
+                source='github',
+                name='AdiAddons/AdiButtonAuras',
+                strategy=Strategies.version,
+                strategy_vals=('2.1.0',),
+            ),
+            Defn(
+                source='github',
+                name='AdiAddons/AdiButtonAuras',
+                strategy=Strategies.version,
+                strategy_vals=('2.0.19',),
+            ),
+            Defn(source='github', name='WeakAuras/WeakAuras2'),
+            Defn(source='github', name='p3lim-wow/Molinari'),
+            Defn(source='github', name='layday/foo-bar'),
         ]
     )
     (
