@@ -2,7 +2,7 @@ from pathlib import Path
 
 import pytest
 
-from instawow.manager import find_base_dirs, should_extract
+from instawow.manager import find_zip_base_dirs, make_zip_member_filter
 from instawow.utils import TocReader, bucketise, merge_intersecting_sets, tabulate
 
 
@@ -11,24 +11,20 @@ def fake_addon():
     yield Path(__file__).parent / 'fixtures' / 'FakeAddon'
 
 
-def test_find_base_dirs_can_find_explicit_dirs():
-    assert find_base_dirs(['b/', 'b/b.toc']) == {'b'}
+def test_find_zip_base_dirs_can_find_explicit_dirs():
+    assert find_zip_base_dirs(['b/', 'b/b.toc']) == {'b'}
 
 
-def test_find_base_dirs_can_find_implicit_dirs():
-    assert find_base_dirs(['b/b.toc']) == {'b'}
+def test_find_zip_base_dirs_can_find_implicit_dirs():
+    assert find_zip_base_dirs(['b/b.toc']) == {'b'}
 
 
-def test_find_base_dirs_discards_resource_forks():
-    assert find_base_dirs(['b/', '__MACOSX/']) == {'b'}
+def test_find_zip_base_dirs_discards_files_in_root():
+    assert find_zip_base_dirs(['a', 'b/b.toc', 'c/']) == {'b', 'c'}
 
 
-def test_find_base_dirs_discards_files_in_root():
-    assert find_base_dirs(['a', 'b/b.toc', 'c/']) == {'b', 'c'}
-
-
-def test_should_extract_discards_names_with_prefix_not_in_dirs():
-    is_member = should_extract({'b'})
+def test_make_zip_member_filter_discards_names_with_prefix_not_in_dirs():
+    is_member = make_zip_member_filter({'b'})
     assert list(map(is_member, ['a/', 'b/', 'aa/', 'bb/'])) == [False, True, False, False]
 
 
@@ -37,13 +33,13 @@ def test_loading_toc_from_path(fake_addon):
     with pytest.raises(FileNotFoundError):
         TocReader.from_path(fake_addon / 'MissingToc.toc')
 
-    TocReader.from_path_name(fake_addon)
+    TocReader.from_parent_folder(fake_addon)
     with pytest.raises(FileNotFoundError):
-        TocReader.from_path_name(fake_addon.parent / 'MissingAddon')
+        TocReader.from_parent_folder(fake_addon.parent / 'MissingAddon')
 
 
 def test_parsing_toc_entries(fake_addon):
-    toc_reader = TocReader.from_path_name(fake_addon)
+    toc_reader = TocReader.from_parent_folder(fake_addon)
     assert toc_reader.entries == {
         'Normal': 'Normal entry',
         'Compact': 'Compact entry',
@@ -51,7 +47,7 @@ def test_parsing_toc_entries(fake_addon):
 
 
 def test_indexing_toc_entries(fake_addon):
-    toc_reader = TocReader.from_path_name(fake_addon)
+    toc_reader = TocReader.from_parent_folder(fake_addon)
     assert toc_reader['Normal'] == ('Normal', 'Normal entry')
     assert toc_reader['Compact'] == ('Compact', 'Compact entry')
     assert toc_reader['Indented'] == ('Indented', '')
@@ -60,7 +56,7 @@ def test_indexing_toc_entries(fake_addon):
 
 
 def test_multiindexing_toc_entries(fake_addon):
-    toc_reader = TocReader.from_path_name(fake_addon)
+    toc_reader = TocReader.from_parent_folder(fake_addon)
     assert toc_reader['Normal', 'Compact'] == ('Normal', 'Normal entry')
     assert toc_reader['Compact', 'Normal'] == ('Compact', 'Compact entry')
     assert toc_reader['Indented', 'Normal'] == ('Normal', 'Normal entry')
@@ -72,7 +68,7 @@ def test_bucketise_bucketises_by_putting_things_in_a_bucketing_bucket():
 
 
 def test_tabulate(fake_addon):
-    toc_reader = TocReader.from_path_name(fake_addon)
+    toc_reader = TocReader.from_parent_folder(fake_addon)
     data = [('key', 'value'), *toc_reader.entries.items()]
     assert (
         tabulate(data)
